/*
  Copyright 2008, UCAR/Unidata
  See COPYRIGHT file for copying and redistribution conditions.

  This program tests the large file bug in netCDF 3.6.2,
  creating byte and short variables larger than 4 GiB.

  $Id: tst_big_var.c,v 1.3 2008/05/07 14:05:05 ed Exp $
*/
#include <config.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nc_tests.h>
#include <netcdf.h>

/* Test with both classic and 64-bit offset files. If netcdf-4 is
 * included, test with both netCDF-4 format variants also. */
#ifdef USE_NETCDF4
#define NUM_FORMATS (4)
#else
#define NUM_FORMATS (2)
#endif

#define NUMDIMS 2		/* rank of each variable in tests */
#define DIM1 2048
#define DIM2 2097153		/* DIM1*DIM2*sizeof(char)   > 2**32 */

/* 
 * In netCDF-3.6.2, an assertion failure occurs on 32-bit platforms
 * when creating a byte variable for which the product of dimensions
 * greater than 2**32.  Check that this bug has been fixed.
 */
static void
test_big_var(const char *testfile) {
    int ncid, varid, dimids[NUMDIMS];
    int res;
    int cflag = NC_CLOBBER;
    nc_type type = NC_BYTE;
    char *typename = "NC_BYTE";
    int i;
    size_t index[NUMDIMS];
    int nval = 99;
    int nval_in;

    if ((res = nc_create(testfile, cflag, &ncid)))
	ERR;
    if ((res = nc_set_fill(ncid, NC_NOFILL, NULL)))
	ERR;
    if ((res = nc_def_dim(ncid, "dim1", DIM1, &dimids[0])))
	ERR;
    if ((res = nc_def_dim(ncid, "dim2", DIM2 - 1, &dimids[1])))
	ERR;
    if ((res = nc_def_var(ncid, "var", type, NUMDIMS, dimids, &varid)))
	ERR;
    if ((res = nc_enddef(ncid)))
	ERR;
    index[0] = DIM1 - 1;
    index[1] = DIM2 - 2;
    if ((res = nc_put_var1_int(ncid, varid, index, &nval)))
	ERR;
    if ((res = nc_close(ncid)))
	ERR;
    if ((res = nc_open(testfile, NC_NOWRITE, &ncid)))
	ERR;
    if ((res = nc_inq_varid(ncid, "var", &varid)))
	ERR;
    if ((res = nc_get_var1_int(ncid, varid, index, &nval_in)))
	ERR;
    if (nval != nval_in)
	ERR;
    if ((res = nc_close(ncid)))
	ERR;
}

#define FILE_NAME "tst_big_var.nc"

int
main(int argc, char **argv) {
    int i;
    char testfile[NC_MAX_NAME + 1];
    /* change the order of these and note failure always occurs only
     * in netcdf-4 formats */
    int formats[] = {
	NC_FORMAT_NETCDF4
	,
	NC_FORMAT_CLASSIC
	,
	NC_FORMAT_64BIT
	,
	NC_FORMAT_NETCDF4_CLASSIC
    };

    printf("*** Testing a very big variable.\n");
    sprintf(testfile, "%s/%s", TEMP_LARGE, FILE_NAME);
    for (i = 0; i < NUM_FORMATS; i++)
   {
       printf("*** switching to netCDF format %d.\n", formats[i]);
       nc_set_default_format(formats[i], NULL);
       printf("*** testing creating a byte variable with > 2**32 values...");
       test_big_var(testfile);
       (void) remove(testfile);
       SUMMARIZE_ERR;
   }
    
    FINAL_RESULTS;
}
